@@ -146,7 +146,7 @@ class IndexDatetime(Index[np.datetime64]):
         # NOTE: remove when min numpy is 1.25
         # NOTE: similar branching as in container_util.apply_binary_operator
         # NOTE: all string will have been converted to dt64, or raise ValueError; comparison to same sized iterables (list, tuple) will result in an array when they are the same size
-        if result is False:  # will never be True
+        if result is False:  # type: ignore  # will never be True
             if (  # type: ignore
                 not other_is_array
                 and hasattr(other, '__len__')
@@ -369,7 +369,7 @@ class IndexYear(IndexDatetime):
             raise LocInvalid(e.args[0]) from None
 
     # ---------------------------------------------------------------------------
-    def to_pandas(self) -> None:
+    def to_pandas(self) -> 'pandas.DatetimeIndex':
         """Return a Pandas Index."""
         raise NotImplementedError(
             'Pandas does not support a year type, and it is ambiguous if a date proxy should be the first of the year or the last of the year.'
@@ -470,7 +470,7 @@ class IndexYearMonth(IndexDatetime):
         )
 
     # ---------------------------------------------------------------------------
-    def to_pandas(self) -> None:
+    def to_pandas(self) -> 'pandas.DatetimeIndex':
         """Return a Pandas Index."""
         raise NotImplementedError(
             'Pandas does not support a year month type, and it is ambiguous if a date proxy should be the first of the month or the last of the month.'
