@@ -1,14 +1,15 @@
 from __future__ import annotations
 
 import re
+import sys
 import types
 import typing
 import warnings
 from collections import deque
 from collections.abc import MutableMapping, Sequence
 from enum import Enum
-from functools import partial, wraps
-from inspect import BoundArguments, Signature
+from functools import partial, reduce, wraps
+from inspect import BoundArguments, Parameter, Signature
 from itertools import chain, repeat
 
 import numpy as np
@@ -22,7 +23,7 @@ from static_frame.core.index_base import IndexBase
 from static_frame.core.index_datetime import IndexDatetime
 from static_frame.core.index_hierarchy import IndexHierarchy
 from static_frame.core.series import Series
-from static_frame.core.util import DTYPE_COMPLEX_KIND, INT_TYPES, TLabel
+from static_frame.core.util import DTYPE_COMPLEX_KIND, INT_TYPES, TLabel, WarningsSilent
 from static_frame.core.yarn import Yarn
 
 TFrameAny = Frame[tp.Any, tp.Any, tp.Unpack[tp.Tuple[tp.Any, ...]]]
@@ -37,6 +38,8 @@ if tp.TYPE_CHECKING:
     TShapeComponent = tp.Union[int, EllipsisType]  # pyright: ignore
     TShapeSpecifier = tp.Tuple[TShapeComponent, ...]
 
+PT_LT_314 = sys.version_info[:2] < (3, 14)
+
 
 def _iter_generic_classes() -> tp.Iterable[tp.Type[tp.Any]]:
     if t := getattr(types, 'GenericAlias', None):
@@ -51,8 +54,9 @@ GENERIC_TYPES = tuple(_iter_generic_classes())
 def _iter_union_classes() -> tp.Iterable[tp.Type[tp.Any]]:
     if t := getattr(types, 'UnionType', None):
         yield t
-    if t := getattr(typing, '_UnionGenericAlias', None):
-        yield t  # pyright: ignore
+    if PT_LT_314:  # deprecated in 3.14
+        if t := getattr(typing, '_UnionGenericAlias', None):
+            yield t  # pyright: ignore
 
 
 UNION_TYPES = tuple(_iter_union_classes())
@@ -122,9 +126,13 @@ def to_name(
     v: tp.Any,
     func_to_str: tp.Callable[..., str] = str,
 ) -> str:
+    # NOTE: could special case union to provide T1 | T2 style display in 3.14
     if is_generic(v):
         name = v.__name__
         s = f'{name}[{", ".join(to_name(q) for q in tp.get_args(v))}]'
+    elif is_union(v):
+        # in 3.14 Union is generic
+        s = f'Union[{", ".join(to_name(q) for q in tp.get_args(v))}]'
     elif isinstance(v, tp.TypeVar):
         # str() gets tilde, __name__ does not have tilde
         if v.__bound__:
@@ -144,10 +152,21 @@ def to_name(
     return s
 
 
-def to_signature(sig: BoundArguments, hints: tp.Mapping[str, tp.Any]) -> str:
+def to_signature(
+    sig: BoundArguments,
+    hints: tp.Mapping[str, tp.Any],
+    parameters: tp.Mapping[str, Parameter],
+) -> str:
     msg = []
     for k in sig.arguments:
-        msg.append(f'{k}: {to_name(hints.get(k, tp.Any))}')
+        if parameters[k].kind == Parameter.VAR_POSITIONAL:
+            p = f'*{k}'
+        elif parameters[k].kind == Parameter.VAR_KEYWORD:
+            p = f'**{k}'
+        else:
+            p = k
+        msg.append(f'{p}: {to_name(hints.get(k, tp.Any))}')
+
     r = to_name(hints.get('return', tp.Any))
     return f'({", ".join(msg)}) -> {r}'
 
@@ -770,12 +789,13 @@ def _value_to_hint(value: tp.Any) -> tp.Any:  # tp._GenericAlias
         if not len(value):
             return value.__class__.__class_getitem__(tp.Any)  # type: ignore[attr-defined]
 
-        # as classes may not be hashable, we key to string name to from a set; this is imperfect
+        # as classes may not be hashable, we key to string name to form a set; this is imperfect
         ut = {v.__class__.__name__: v.__class__ for v in value}
         if len(ut) == 1:
             return value.__class__.__class_getitem__(ut[next(iter(ut.keys()))])  # type: ignore[attr-defined]
 
-        hu = tp.Union.__getitem__(tuple(ut.values()))  # pyright: ignore
+        hu = reduce(lambda x, y: x | y, ut.values())
+        # hu = tp.Union.__getitem__(tuple(ut.values()))  # pyright: ignore
         return value.__class__.__class_getitem__(hu)  # type: ignore[attr-defined]
 
     if isinstance(value, MutableMapping):
@@ -788,12 +808,14 @@ def _value_to_hint(value: tp.Any) -> tp.Any:  # tp._GenericAlias
         if len(keys_ut) == 1:
             kt = keys_ut[next(iter(keys_ut.keys()))]
         else:
-            kt = tp.Union.__getitem__(tuple(keys_ut.values()))  # pyright: ignore
+            kt = reduce(lambda x, y: x | y, keys_ut.values())
+            # kt = tp.Union.__getitem__(tuple(keys_ut.values()))  # pyright: ignore
 
         if len(values_ut) == 1:
             vt = values_ut[next(iter(values_ut.keys()))]
         else:
-            vt = tp.Union.__getitem__(tuple(values_ut.values()))  # pyright: ignore
+            vt = reduce(lambda x, y: x | y, values_ut.values())
+            # vt = tp.Union.__getitem__(tuple(values_ut.values()))  # pyright: ignore
 
         return value.__class__.__class_getitem__((kt, vt))  # type: ignore[attr-defined]
 
@@ -1396,7 +1418,9 @@ class TypeVarState:
             if i in self._bound_unset and _check(value, hint).validated:
                 components[i] = _value_to_hint(value)
                 self._bound_unset.discard(i)
-        self._bound = tp.Union.__getitem__(tuple(components))  # pyright: ignore
+        ut = reduce(lambda x, y: x | y, components)
+        self._bound = ut
+        # self._bound = tp.Union.__getitem__(tuple(components))  # pyright: ignore
 
     @property
     def constraints(self) -> tp.Any:
@@ -1431,8 +1455,10 @@ class TypeVarRegistry:
             tvs = TypeVarState(var, value)
             self._id_to_var[var] = tvs
             if hints := tvs.constraints:
-                # with constratings we select one option and use it for the life of the Typevar; on the first value, check that the value meets the constraints (recast as a uion); subsequent checks will be based on the stored value
-                yield value, tp.Union.__getitem__(hints), parent_hints, pv_next  # pyright: ignore
+                # with constratings we select one option and use it for the life of the Typevar; on the first value, check that the value meets the constraints (recast as a union); subsequent checks will be based on the stored value
+                ut = reduce(lambda x, y: x | y, hints)
+
+                yield value, ut, parent_hints, pv_next  # pyright: ignore
         else:
             tvs = self._id_to_var[var]
 
@@ -1688,13 +1714,29 @@ def _check_interface(
     error_action: ErrorAction,
     category: tp.Type[Warning] = UserWarning,
 ) -> tp.Any:
+    sig = Signature.from_callable(func)
+    parameters = sig.parameters
+
     # include_extras insures that Annotated generics are returned
     hints = tp.get_type_hints(func, include_extras=True)
 
-    sig = Signature.from_callable(func)
+    for k, v in parameters.items():
+        # hints applied to VAR_POSITIONAL are not auto upgraded to a tuple, while argument values are
+        h = hints.get(k, tp.Any)
+        if v.kind == Parameter.VAR_POSITIONAL:
+            if tp.get_origin(h) is not tuple:
+                hints[k] = tuple[h, ...]  # type: ignore
+        elif v.kind == Parameter.VAR_KEYWORD:
+            if isinstance(h, tp._TypedDictMeta):  # type: ignore
+                pass
+            elif tp.get_origin(h) is not dict:
+                hints[k] = dict[str, h]  # type: ignore
+
+    # if an arg is VAR_POSITIONAL, it will be bound here as a tuple
     sig_bound = sig.bind(*args, **kwargs)
     sig_bound.apply_defaults()
-    sig_str = to_signature(sig_bound, hints)
+
+    sig_str = to_signature(sig_bound, hints, parameters)
     parent_hints = (f'args of {sig_str}',)
     parent_values = (func,)
 
